import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query';
import { HttpVerb, makeApiCall } from '../services';
import { PaginationMethod } from '../types/common-types';
/*
 * Function to get the data of a table
 * @param endPoint: string
 * @param page: number
 * @param limit: number
 * @returns {Promise}
 */
const getTableData = async (_, page, limit, sortBy = '', order = 'asc', filters, apiURL, search_term = '', paginationMethod) => {
    const params = {
        ...(paginationMethod === PaginationMethod.PAGE && page && { page }),
        ...(paginationMethod === PaginationMethod.OFFSET && page && { offset: page - 1 }),
        ...(limit && { limit }),
        ...(search_term && { search_term }),
        ...(sortBy && { sortBy }),
        ...(order && { order }),
        ...filters,
    };
    const result = await makeApiCall(HttpVerb.GET, apiURL, undefined, {
        params,
    });
    return {
        details: result?.data?.data ?? [],
        totalRecords: result?.data?.total_count ?? 0,
    };
};
/*
 * Function to delete an item
 * @param id: number | string
 * @param apiURL: string
 */
const deleteItem = async (id, apiURL) => {
    const result = await makeApiCall(HttpVerb.DELETE, `${apiURL}/${id}`);
    return result;
};
/*
 * Function to add an item
 * @param data: any
 * @param apiURL: string
 */
const addItem = async (data, apiURL) => {
    const result = await makeApiCall(HttpVerb.POST, apiURL, data);
    return result;
};
/*
 * Function to update an item
 * @param id: number | string
 * @param data: any
 * @param apiURL: string
 */
const updateItem = async (id, data, apiURL) => {
    const result = await makeApiCall(HttpVerb.PATCH, `${apiURL}/${id}`, data);
    return result;
};
const updateStatus = async (id, apiURL) => {
    const result = await makeApiCall(HttpVerb.PATCH, `${apiURL}/${id}/activate`);
    return result;
};
const useTableData = (props) => {
    const { key = '', page = 1, limit = 10, sortBy = '', order = '', filters = {}, apiEndPoint = {}, search_term = '', paginationMethod = PaginationMethod.PAGE, handleDetailsSubmit, } = props;
    const queryClient = useQueryClient();
    // Determine the API URLs based on available endpoints
    const apiURL = apiEndPoint.ALL || `${import.meta.env.VITE_API_END_POINT}/${key}`;
    const getApiURL = apiEndPoint.GET || apiURL;
    const postApiURL = apiEndPoint.POST || apiURL;
    const patchApiURL = apiEndPoint.PATCH || apiURL;
    const deleteApiURL = apiEndPoint.DELETE || apiURL;
    const toggleApiURL = apiEndPoint.TOGGLE || apiURL;
    const query = useQuery({
        queryKey: [key, page, limit, sortBy, order, filters, search_term, paginationMethod],
        queryFn: () => getTableData(key, page, limit, sortBy, order, filters, getApiURL, search_term, paginationMethod),
    });
    const deleteMutation = useMutation({
        mutationFn: async ({ id }) => {
            await deleteItem(id, deleteApiURL);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Delete item failed:', error);
        },
    });
    const addMutation = useMutation({
        mutationFn: async ({ data }) => {
            const updatedData = handleDetailsSubmit ? await handleDetailsSubmit(data) : data;
            await addItem(updatedData, postApiURL);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Add item failed:', error);
        },
    });
    const updateMutation = useMutation({
        mutationFn: async ({ id, data }) => {
            const updatedData = handleDetailsSubmit ? await handleDetailsSubmit(data) : data;
            await updateItem(id, updatedData, patchApiURL);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Update item failed:', error);
        },
    });
    const updateStatusMutation = useMutation({
        mutationFn: async ({ id }) => {
            await updateStatus(id, toggleApiURL);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Update status failed:', error);
        },
    });
    // Function to add new item to the table
    const addNewItem = (data, onSuccess, onError) => {
        addMutation.mutate({ data }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to update the selected item
    const updateSelectedItem = (id, data, onSuccess, onError) => {
        updateMutation.mutate({ id, data }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to delete the selected item
    const deleteSelectedItem = (id, onSuccess, onError) => {
        deleteMutation.mutate({ id }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to update the status of the item
    const updateItemStatus = (id, onSuccess, onError) => {
        updateStatusMutation.mutate({ id }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    return {
        ...query,
        isLoading: query.isLoading,
        isPending: addMutation.isPending || updateMutation.isPending || deleteMutation.isPending,
        addItem: addNewItem,
        deleteItem: deleteSelectedItem,
        updateItem: updateSelectedItem,
        updateStatus: updateItemStatus,
    };
};
export { useTableData };
